/**
 * This work was created by participants in the DataONE project, and is
 * jointly copyrighted by participating institutions in DataONE. For 
 * more information on DataONE, see our web site at http://dataone.org.
 *
 *   Copyright ${year}
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 * 
 * $Id$
 */
package org.dataone.tidy.dao;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.time.FastDateFormat;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dataone.configuration.Settings;
import org.dataone.service.types.v1.SystemMetadata;
import org.dataone.service.util.TypeMarshaller;
import org.hibernate.annotations.Index;
import org.jibx.runtime.JiBXException;

/**
 * MergeResult represents the output of a system metadata merge process.  It records
 * the state of the Member Nodes system metadata document before the merge occured.  It
 * also records the state of the system metadata document after the merge - which represent
 * the new version of the system metadata which will replace the system metadata document currently
 * in use (on CN and MN).
 * 
 * MergeResult is configured via Spring framework as a spring-data JPA object and
 * is stored in relational datastore.
 * 
 * @author sroseboo
 *
 */

@Entity
@Table(name = "merge_result")
public class MergeResult implements Serializable {

    private static final long serialVersionUID = 995840092901314022L;
    private static final Long RUN_ID = Settings.getConfiguration().getLong("tidy.run.id", 999);

    @Transient
    private static final Log log = LogFactory.getLog(MergeResult.class);

    @Transient
    private final FastDateFormat format = FastDateFormat.getInstance("MM/dd/yyyy:HH:mm:ss:SS");

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @Index(name = "index_pid")
    @Column(columnDefinition = "TEXT", nullable = false)
    private String pid;

    @Index(name = "index_run")
    @Column
    private Long runId = RUN_ID;

    private Date mergeDate;

    @Column(columnDefinition = "TEXT")
    private String mnSystemMetadataStr;

    @Column(columnDefinition = "TEXT")
    private String mergedSystemMetadataStr;

    @Column(columnDefinition = "TEXT")
    private String processingException;

    @Column(nullable = false)
    private String mergeStatusStr;
    
    @Column
    private String review;
    
    @Column
    private String authMN;

    // default constructor required by jpa
    public MergeResult() {
    }

    public MergeResult(SystemMetadata mergedSystemMetadata, SystemMetadata mnSystemMetadata,
            MergeStatus status) {
        this.mergedSystemMetadataStr = marshalSystemMetadata(mergedSystemMetadata);
        this.mnSystemMetadataStr = marshalSystemMetadata(mnSystemMetadata);
        this.pid = mergedSystemMetadata.getIdentifier().getValue();
        setMergeStatus(status);
    }

    public MergeResult(String pidValue, MergeStatus status) {
        this.pid = pidValue;
        setMergeStatus(status);
        this.mergeDate = new Date(System.currentTimeMillis());
    }

    @Transient
    private SystemMetadata unMarshalSystemMetadata(String systemMetadataString) {
        InputStream is = new ByteArrayInputStream(systemMetadataString.getBytes());
        SystemMetadata smd = null;
        try {
            smd = TypeMarshaller.unmarshalTypeFromStream(SystemMetadata.class, is);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } catch (InstantiationException e) {
            log.error(e.getMessage(), e);
        } catch (IllegalAccessException e) {
            log.error(e.getMessage(), e);
        } catch (JiBXException e) {
            log.error(e.getMessage(), e);
        }
        return smd;
    }

    @Transient
    public static String marshalSystemMetadata(SystemMetadata smd) {
        if (smd == null) {
            return null;
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            TypeMarshaller.marshalTypeToOutputStream(smd, os);
        } catch (JiBXException jibxEx) {
            log.error(jibxEx.getMessage(), jibxEx);
        } catch (IOException ioEx) {
            log.error(ioEx.getMessage(), ioEx);
        }
        String sysMetaStr = null;
        try {
            sysMetaStr = os.toString("UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage());
        }
        return sysMetaStr;
    }

    public Long getId() {
        return id;
    }

    public String getPid() {
        return pid;
    }

    public void setPid(String pid) {
        this.pid = pid;
    }

    public Date getMergeDate() {
        return mergeDate;
    }

    @Transient
    public String getMergeDateString() {
        return this.format.format(this.mergeDate);
    }

    public void setMergeDate(Date mergeDate) {
        this.mergeDate = mergeDate;
    }

    public String getMergedSystemMetadataStr() {
        return mergedSystemMetadataStr;
    }

    public void setMergedSystemMetadataStr(String systemMetadata) {
        this.mergedSystemMetadataStr = systemMetadata;
    }

    public String getMnSystemMetadataStr() {
        return mnSystemMetadataStr;
    }

    public void setMnSystemMetadataStr(String mnSystemMetadataStr) {
        this.mnSystemMetadataStr = mnSystemMetadataStr;
    }

    public SystemMetadata getMergedSystemMetadata() {
        if (this.mergedSystemMetadataStr == null) {
            return null;
        }
        return this.unMarshalSystemMetadata(this.mergedSystemMetadataStr);
    }

    public SystemMetadata getMnSystemMetadata() {
        if (this.mnSystemMetadataStr == null) {
            return null;
        }
        return this.unMarshalSystemMetadata(this.mnSystemMetadataStr);
    }

    public void setProcessingException(String exception) {
        this.processingException = exception;
    }

    public String getProcessingException() {
        return processingException;
    }

    public void setMergeStatusStr(String status) {
        this.mergeStatusStr = status;
    }

    public String getMergeStatusStr() {
        return this.mergeStatusStr;
    }

    @Transient
    public void setMergeStatus(MergeStatus status) {
        this.mergeStatusStr = status.toString();
    }

    @Transient
    public MergeStatus getMergeStatus() {
        return MergeStatus.valueOf(this.mergeStatusStr);
    }

    @Transient
    public void setMnSystemMetadata(SystemMetadata sysmeta) {
        setMnSystemMetadataStr(MergeResult.marshalSystemMetadata(sysmeta));
    }

    @Transient
    public void setMergedSystemMetadata(SystemMetadata sysmeta) {
        setMergedSystemMetadataStr(MergeResult.marshalSystemMetadata(sysmeta));
    }

    public Long getRunId() {
        return runId;
    }

    public void setRunId(Long runId) {
        this.runId = runId;
    }
    
    public String getReview() {
        return this.review;
    }

    public void setReview(String review) {
        this.review = review;
    }
    
    public String getAuthoritativeMemberNode() {
        return this.authMN;
    }

    public void setAuthoritativeMemberNode(String memberNode) {
        this.authMN = memberNode;
    }

    @Override
    public String toString() {
        return "MergeResult [id=" + id + ", pid=" + pid + ", mergeDate=" + getMergeDateString()
                + ", runId=" + runId + ", status=" + mergeStatusStr + ", authMN="+ authMN + ", exception='"+
        		getProcessingException() + "']";
    }

}
