#!/bin/bash

# This work was created by participants in the DataONE project, and is
# jointly copyrighted by participating institutions in DataONE. For
# more information on DataONE, see our web site at http://dataone.org.
#
#   Copyright 2014
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Dumps tidy merged metadata related tables from the Coordinating Node that the script is run on into a
# .sql dump file.
# 2) Modifies the SQL dump to remove a 'merged_' prefix to entity names 
# 3) Transfers the SQL dump in bulk via rsync to the target machine where the dump will be restored
# to create the new per-CN tables where the tidy merging will happen.
# Note that transferring  and running the SQL dump script on the target machine is a manual step.

RTN_JOIN="";
function join() {
        RTN_JOIN="";
        local exprString=$1
        shift
        local -a listFields=($@)
        tmpJoin=$(printf "${exprString}%s" "${listFields[@]}")
        RTN_JOIN=${tmpJoin:1}
        RTN_JOIN="${RTN_JOIN#"${RTN_JOIN%%[![:space:]]*}"}" # remove leading whitespace characters
        RTN_JOIN="${RTN_JOIN%"${RTN_JOIN##*[![:space:]]}"}" # remove trailing whitespace characters
        return
}


DUMP_DB="metacat";
DUMP_DIR="/var/lib/postgresql/pg_tidy_dump";

if [[ ! -d ${DUMP_DIR} ]]; then
    mkdir ${DUMP_DIR};
    chown -R postgres.postgres ${DUMP_DIR};
fi

DUMP_FILE="${DUMP_DIR}/dump-merged-$(date +%Y%m%d%H%M).sql";
DUMP_CMD="pg_dump \
-h localhost \
-w \
-t merged_smreplicationpolicy \
-t merged_smreplicationstatus \
-t merged_systemmetadata \
-t merged_xml_access \
-f ${DUMP_FILE} \
-U metacat d1-tidy";
echo "Running: " "${DUMP_CMD}";
su postgres -c "${DUMP_CMD}"

echo "Altering SQL DDL statements ...";
echo "";

sed -i.bak -r -e "s/\(principal_name\)\:\:text/principal_name/" ${DUMP_FILE};
sed -i.bak -r -e "s/timestamp\ without\ time\ zone/TIMESTAMP/" ${DUMP_FILE};
sed -i.bak -r -e "s/character\ varying/VARCHAR/" ${DUMP_FILE};
sed -i.bak -r -e "s/bigint/INT8/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE merged_systemmetadata/DROP TABLE IF EXISTS systemmetadata; CREATE TABLE systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE merged_systemmetadata/ALTER TABLE systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.merged_systemmetadata/ALTER TABLE public\.systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY merged_systemmetadata/ALTER TABLE ONLY systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/COPY merged_systemmetadata/COPY systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT merged_systemmetadata/ADD CONSTRAINT systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX merged_systemmetadata/CREATE INDEX systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/ON merged_systemmetadata/ON systemmetadata/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES merged_systemmetadata/REFERENCES systemmetadata/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE merged_smreplicationpolicy/DROP TABLE IF EXISTS smreplicationpolicy; CREATE TABLE smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE merged_smreplicationpolicy/ALTER TABLE smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.merged_smreplicationpolicy/ALTER TABLE public\.smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY merged_smreplicationpolicy/ALTER TABLE ONLY smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/COPY merged_smreplicationpolicy/COPY smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT merged_smreplicationpolicy/ADD CONSTRAINT smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX merged_smreplicationpolicy/CREATE INDEX smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/ON merged_smreplicationpolicy/ON smreplicationpolicy/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES merged_smreplicationpolicy/REFERENCES smreplicationpolicy/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE merged_smreplicationstatus/DROP TABLE IF EXISTS smreplicationstatus; CREATE TABLE smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE merged_smreplicationstatus/ALTER TABLE smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.merged_smreplicationstatus/ALTER TABLE public\.smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY merged_smreplicationstatus/ALTER TABLE ONLY smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/COPY merged_smreplicationstatus/COPY smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT merged_smreplicationstatus/ADD CONSTRAINT smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX merged_smreplicationstatus/CREATE INDEX smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/ON merged_smreplicationstatus/ON smreplicationstatus/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES merged_smreplicationstatus/REFERENCES smreplicationstatus/" ${DUMP_FILE};

sed -i.bak -e "s/CREATE TABLE merged_xml_access/DROP TABLE IF EXISTS xml_access; CREATE TABLE xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE merged_xml_access/ALTER TABLE xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE public\.merged_xml_access/ALTER TABLE public\.xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ALTER TABLE ONLY merged_xml_access/ALTER TABLE ONLY xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/COPY merged_xml_access/COPY xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ADD CONSTRAINT merged_xml_access/ADD CONSTRAINT xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/CREATE INDEX merged_xml_access/CREATE INDEX xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/ON merged_xml_access/ON xml_access/" ${DUMP_FILE};
sed -i.bak -e "s/REFERENCES merged_xml_access/REFERENCES xml_access/" ${DUMP_FILE};

D1_CONF_DIR=/etc/dataone
D1_DEB_CONF_XML="d1DebConfig.xml"

MY_POSSIBLE_IPS=(`hostname --all-ip-addresses`)
ALL_ENVIRONMENT_IPS=(`xmlstarlet sel -t --match "//environment[@context != 'CUSTOM']/machine"  --value-of "concat(@ip, ' ')"  "${D1_CONF_DIR}/${D1_DEB_CONF_XML}"`)

for i in ${MY_POSSIBLE_IPS[@]}
	do
		if [[ "${ALL_ENVIRONMENT_IPS[@]}" =~ "${i}" ]]; then
			IP_ADDRESS=${i}
		fi
	done
CN_CONTEXT_LIST=(`xmlstarlet sel -t --match "//machine[@ip='${IP_ADDRESS}']"  --value-of " string('dataone-cn-os-core/cn.context.label ')" --value-of "parent::node()/@context" "${D1_CONF_DIR}/${D1_DEB_CONF_XML}"`)
		
# Set the global context variable
CONTEXT=${CN_CONTEXT_LIST[1]}
CN_HOSTNAME=(`xmlstarlet sel -t --match "//environment[@context='${CONTEXT}']/machine[@ip='${IP_ADDRESS}']/question[@keyref='dataone-cn-os-core/cn.hostname']"  --value-of "concat(text(), ' ')"  "$D1_CONF_DIR/${D1_DEB_CONF_XML}"`)

echo $CN_HOSTNAME

CN_HOSTNAMELIST=(`xmlstarlet sel -t --match "//environment[@context='${CONTEXT}']/machine/question[@keyref='dataone-cn-os-core/cn.hostname']"  --value-of "concat(text(), ' ')"  "$D1_CONF_DIR/${D1_DEB_CONF_XML}"`)
join ' ' ${CN_HOSTNAMELIST[@]}
HOSTNAMELIST=${RTN_JOIN}

TX_HOSTNAMELIST=${HOSTNAMELIST/$CN_HOSTNAME}


for host in ${TX_HOSTNAMELIST[@]}
        do
                echo "transfering ${DUMP_FILE} to ${host}"
		su - postgres -c "ssh ${host} mkdir -p ${DUMP_DIR}"
		su - postgres -c "rsync --progress --partial -e ssh ${DUMP_FILE} ${host}:${DUMP_DIR}"
        done

